`timescale 1ns / 1ps
////////////////////////////////////////////////////////////////////////////////
// Engineer:       Michael Siegenthaler
//
// Module Name:    main
// Project Name:   Parallel Port Interface Demo
// Revision:       October 25, 2005
//
////////////////////////////////////////////////////////////////////////////////
//
// On the Digilent Pegasus, the switches are used to enter a memory
// address. This address is displayed on the right two digits of the seven
// segment display. The byte stored at the address is displayed on the left
// two digits of the seven segment display.
//
////////////////////////////////////////////////////////////////////////////////
module main(mclk, btn, sw, leds, segs, an, ac,
  ctrl_write, ctrl_dstb, ctrl_rst, ctrl_astb, sta_wait, data_bus);

  `include "hex7seg.v"

  // inputs/outputs
  input mclk;
  input [3:0] btn;
  input [7:0] sw;
  output [7:0] leds;
  output [0:7] segs;
  output [3:0] an;
  output [3:0] ac;
  input ctrl_write;
  input ctrl_dstb;
  input ctrl_rst;
  input ctrl_astb;
  output sta_wait;
  inout [7:0] data_bus;

  // external nodes
  wire mclk;
  wire [3:0] btn;
  wire [7:0] sw;
  reg [7:0] sw2;
  wire [7:0] leds;
  wire [0:7] segs;
  wire [3:0] an;
  wire [3:0] ac;

  // assign ac to avoid error on synthesize
  assign ac = 4'b0;

  // parallel port lines
  wire ctrl_write;
  wire ctrl_dstb;
  wire ctrl_rst;
  wire ctrl_astb;
  wire sta_wait;
  wire [7:0] data_bus;

  // RAM connections
  wire ram_clk_a, ram_en_a, ram_we_a;
  wire [7:0] ram_addr_a, ram_di_a, ram_do_a;
  wire ram_clk_b, ram_en_b, ram_we_b;
  wire [7:0] ram_addr_b, ram_di_b, ram_do_b;


  assign ram_clk_b = mclk;
  assign ram_en_b = 1'b1;
  assign ram_we_b = 1'b0;
  assign ram_addr_b = sw2;
  assign ram_di_b = ram_di_a;
  assign leds = ram_do_b;

  // Register the switch inputs
  always @(posedge mclk) begin
    sw2 <= sw;
  end

  // seven segments
  wire [0:7] segs0, segs1, segs2, segs3;
  assign segs3[0:6] = hex7seg(leds[7:4]);
  assign segs2[0:6] = hex7seg(leds[3:0]);
  assign segs1[0:6] = hex7seg(sw2[7:4]);
  assign segs0[0:6] = hex7seg(sw2[3:0]);

  assign segs0[7] = 1'b1;
  assign segs1[7] = 1'b1;
  assign segs2[7] = 1'b1;
  assign segs3[7] = 1'b1;

  // display driver
  segdrive i_segdrive (mclk, segs3, segs2, segs1, segs0, segs, an);

  // RAM
  ioram i_ioram (ram_clk_a, ram_addr_a, ram_en_a, ram_we_a, ram_di_a, ram_do_a,
                 ram_clk_b, ram_addr_b, ram_en_b, ram_we_b, ram_di_b, ram_do_b);

  // EPP I/O Slave
  eppioslave i_eppioslave (mclk, ctrl_write, ctrl_dstb, ctrl_rst, ctrl_astb, sta_wait, data_bus,
                           ram_clk_a, ram_addr_a, ram_en_a, ram_we_a, ram_di_a, ram_do_a);

endmodule
